@@ -22,8 +22,8 @@
 #define _GNOME_WP_INFO_H_
 
 #include <glib.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomeui/gnome-thumbnail.h>
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnomeui/gnome-desktop-thumbnail.h>
 
 typedef struct _GnomeWPInfo GnomeWPInfo;
 
@@ -33,13 +33,13 @@ struct _GnomeWPInfo {
   gchar * name;
   gchar * mime_type;
 
-  GnomeVFSFileSize size;
+  goffset size;
 
   time_t mtime;
 };
 
 GnomeWPInfo * gnome_wp_info_new (const gchar * uri,
-				 GnomeThumbnailFactory * thumbs);
+				 GnomeDesktopThumbnailFactory * thumbs);
 void gnome_wp_info_free (GnomeWPInfo * info);
 
 #endif
