@@ -26,7 +26,8 @@
 #  include <config.h>
 #endif
 
-#include <gnome.h>
+#include <glib/gi18n.h>
+#include <string.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 
@@ -34,6 +35,7 @@
 
 #include "gnome-keyboard-properties-xkb.h"
 
+static GladeXML *chooser_dialog = NULL;
 static const char *current1st_level_id = NULL;
 static GtkWidget *current_vbox = NULL;
 static GtkWidget *current_none_radio = NULL;
@@ -94,6 +96,13 @@ xkb_options_expander_selcounter_add (GtkWidget * expander, int value)
 			    + value));
 }
 
+static void
+xkb_options_expander_selcounter_reset (GtkWidget * expander)
+{
+	g_object_set_data (G_OBJECT (expander), SELCOUNTER_PROP,
+			   GINT_TO_POINTER (0));
+}
+
 static void
 xkb_options_expander_highlight (GtkWidget * expander)
 {
@@ -170,6 +179,20 @@ xkb_options_is_selected (gchar * optionname)
 	return retval;
 }
 
+/* Make sure selected options stay visible when navigating with the keyboard */
+static gboolean
+option_focused_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+{
+	GtkScrolledWindow *win = GTK_SCROLLED_WINDOW (data);
+	GtkAllocation *alloc = &widget->allocation;
+	GtkAdjustment *adj;
+
+	adj = gtk_scrolled_window_get_vadjustment (win);
+	gtk_adjustment_clamp_page (adj, alloc->y, alloc->y + alloc->height);
+
+	return FALSE;
+}
+
 /* Update xkb backend to reflect the new UI state */
 static void
 option_toggled_cb (GtkWidget * checkbutton, gpointer data)
@@ -219,6 +242,10 @@ xkb_options_add_option (XklConfigRegistry * config_registry,
 			    gtk_radio_button_get_group (GTK_RADIO_BUTTON
 							(option_check));
 			current_none_radio = option_check;
+
+			g_signal_connect (option_check, "focus-in-event",
+					  G_CALLBACK (option_focused_cb),
+					  WID ("options_scroll"));
 		}
 		option_check =
 		    gtk_radio_button_new_with_label (current_radio_group,
@@ -239,9 +266,13 @@ xkb_options_add_option (XklConfigRegistry * config_registry,
 	g_object_set_data_full (G_OBJECT (option_check), OPTION_ID_PROP,
 				full_option_name, g_free);
 
-	g_signal_connect (G_OBJECT (option_check), "toggled",
+	g_signal_connect (option_check, "toggled",
 			  G_CALLBACK (option_toggled_cb), NULL);
 
+	g_signal_connect (option_check, "focus-in-event",
+			  G_CALLBACK (option_focused_cb),
+			  WID ("options_scroll"));
+
 	gtk_box_pack_start_defaults (GTK_BOX (current_vbox), option_check);
 
 	xkb_options_expander_selcounter_add (xkb_options_get_expander
@@ -273,6 +304,8 @@ xkb_options_add_group (XklConfigRegistry * config_registry,
 	gtk_expander_set_use_markup (GTK_EXPANDER (expander), TRUE);
 	g_object_set_data_full (G_OBJECT (expander), "utfGroupName",
 				utf_group_name, g_free);
+	g_object_set_data_full (G_OBJECT (expander), "groupId",
+				g_strdup (config_item->name), g_free);
 
 	g_free (titlemarkup);
 	align = gtk_alignment_new (0, 0, 1, 1);
@@ -297,6 +330,10 @@ xkb_options_add_group (XklConfigRegistry * config_registry,
 	expanders_list = g_slist_append (expanders_list, expander);
 	g_object_set_data (G_OBJECT (dialog), EXPANDERS_PROP,
 			   expanders_list);
+
+	g_signal_connect (expander, "focus-in-event",
+			  G_CALLBACK (option_focused_cb),
+			  WID ("options_scroll"));
 }
 
 static gint
@@ -304,9 +341,9 @@ xkb_options_expanders_compare (GtkWidget * expander1,
 			       GtkWidget * expander2)
 {
 	const gchar *t1 =
-	    gtk_expander_get_label (GTK_EXPANDER (expander1));
+	    g_object_get_data (G_OBJECT (expander1), "utfGroupName");
 	const gchar *t2 =
-	    gtk_expander_get_label (GTK_EXPANDER (expander2));
+	    g_object_get_data (G_OBJECT (expander2), "utfGroupName");
 	return g_utf8_collate (t1, t2);
 }
 
@@ -335,6 +372,8 @@ xkb_options_load_options (GladeXML * dialog)
 	expanders_list =
 	    g_slist_sort (expanders_list,
 			  (GCompareFunc) xkb_options_expanders_compare);
+	g_object_set_data (G_OBJECT (dialog), EXPANDERS_PROP,
+			   expanders_list);
 	while (expanders_list) {
 		expander = GTK_WIDGET (expanders_list->data);
 		gtk_box_pack_start (GTK_BOX (opts_vbox), expander, FALSE,
@@ -342,43 +381,62 @@ xkb_options_load_options (GladeXML * dialog)
 		expanders_list = expanders_list->next;
 	}
 
-	/* just cleanup */
-	expanders_list =
-	    g_object_get_data (G_OBJECT (dialog), EXPANDERS_PROP);
-	g_object_set_data (G_OBJECT (dialog), EXPANDERS_PROP, NULL);
-	g_slist_free (expanders_list);
-
 	gtk_widget_show_all (opts_vbox);
 }
 
 static void
-chooser_response_cb (GtkDialog *dialog, gint response, gpointer data)
+chooser_response_cb (GtkDialog * dialog, gint response, gpointer data)
 {
-	if (response == GTK_RESPONSE_CLOSE)
+	if (response == GTK_RESPONSE_CLOSE) {
+		/* just cleanup */
+		GSList *expanders_list =
+		    g_object_get_data (G_OBJECT (dialog), EXPANDERS_PROP);
+		g_object_set_data (G_OBJECT (dialog), EXPANDERS_PROP,
+				   NULL);
+		g_slist_free (expanders_list);
+
 		gtk_widget_destroy (GTK_WIDGET (dialog));
+		chooser_dialog = NULL;
+	}
 }
 
-/* Create popup dialog*/
+/* Create popup dialog */
 void
 xkb_options_popup_dialog (GladeXML * dialog)
 {
-	GladeXML *chooser_dialog =
+	GtkWidget *chooser;
+
+	chooser_dialog =
 	    glade_xml_new (GNOMECC_GLADE_DIR
 			   "/gnome-keyboard-properties.glade",
 			   "xkb_options_dialog", NULL);
-	GtkWidget *chooser = CWID ("xkb_options_dialog");
 
+	chooser = CWID ("xkb_options_dialog");
 	gtk_window_set_transient_for (GTK_WINDOW (chooser),
 				      GTK_WINDOW (WID
 						  ("keyboard_dialog")));
 	xkb_options_load_options (chooser_dialog);
 
 	g_signal_connect (chooser, "response",
-	    G_CALLBACK (chooser_response_cb), dialog);
+			  G_CALLBACK (chooser_response_cb), dialog);
 
 	gtk_dialog_run (GTK_DIALOG (chooser));
 }
 
+/* Update selected option counters for a group-bound expander */
+static void
+xkb_options_update_option_counters (XklConfigRegistry * config_registry,
+				    XklConfigItem * config_item,
+				    GtkWidget * expander)
+{
+	gchar *full_option_name =
+	    g_strdup (gkbd_keyboard_config_merge_items
+		      (current1st_level_id, config_item->name));
+	gboolean current_state =
+	    xkb_options_is_selected (full_option_name);
+	xkb_options_expander_selcounter_add (expander, current_state);
+}
+
 /* Respond to a change in the xkb gconf settings */
 static void
 xkb_options_update (GConfClient * client,
@@ -388,6 +446,27 @@ xkb_options_update (GConfClient * client,
 	   This is here to avoid calling it N_OPTIONS times for each gconf
 	   change. */
 	enable_disable_restoring (dialog);
+
+	if (chooser_dialog != NULL) {
+		GSList *expanders_list =
+		    g_object_get_data (G_OBJECT (chooser_dialog),
+				       EXPANDERS_PROP);
+		while (expanders_list) {
+			GtkWidget *expander =
+			    GTK_WIDGET (expanders_list->data);
+			gchar *group_id =
+			    g_object_get_data (G_OBJECT (expander),
+					       "groupId");
+			current1st_level_id = group_id;
+			xkb_options_expander_selcounter_reset (expander);
+			xkl_config_registry_foreach_option
+			    (config_registry, group_id,
+			     (ConfigItemProcessFunc)
+			     xkb_options_update_option_counters, expander);
+			xkb_options_expander_highlight (expander);
+			expanders_list = expanders_list->next;
+		}
+	}
 }
 
 void
