@@ -25,8 +25,8 @@
 
 #include <config.h>
 
+#include <glib/gi18n.h>
 #include <string.h>
-#include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <gdk/gdkx.h>
@@ -46,42 +46,6 @@
 #include <X11/Xcursor/Xcursor.h>
 #endif
 
-/******************************************************************************/
-/* A quick custom widget to ensure that the left handed toggle works no matter
- * which button is pressed.
- */
-typedef struct { GtkCheckButton parent; } MouseCappletCheckButton;
-typedef struct { GtkCheckButtonClass parent; } MouseCappletCheckButtonClass;
-GNOME_CLASS_BOILERPLATE (MouseCappletCheckButton, mouse_capplet_check_button,
-			 GtkCheckButton, GTK_TYPE_CHECK_BUTTON)
-static void mouse_capplet_check_button_instance_init (MouseCappletCheckButton *obj) { }
-
-static gboolean
-mouse_capplet_check_button_button_press (GtkWidget *widget, GdkEventButton *event)
-{
-	if (event->type == GDK_BUTTON_PRESS) {
-		if (!GTK_WIDGET_HAS_FOCUS (widget))
-			gtk_widget_grab_focus (widget);
-		gtk_button_pressed (GTK_BUTTON (widget));
-	}
-	return TRUE;
-}
-static gboolean
-mouse_capplet_check_button_button_release (GtkWidget *widget, GdkEventButton *event)
-{
-      gtk_button_released (GTK_BUTTON (widget));
-      return TRUE;
-}
-
-static void
-mouse_capplet_check_button_class_init (MouseCappletCheckButtonClass *klass)
-{
-	GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
-	widget_class->button_press_event   = mouse_capplet_check_button_button_press;
-	widget_class->button_release_event = mouse_capplet_check_button_button_release;
-}
-/******************************************************************************/
-
 enum
 {
 	DOUBLE_CLICK_TEST_OFF,
@@ -254,7 +218,7 @@ event_box_button_press_event (GtkWidget   *widget,
 	static gint                test_on_timeout_id     = 0;
 	static gint                test_maybe_timeout_id  = 0;
 	static guint32             double_click_timestamp = 0;
-	GtkWidget                  *image;
+	GtkWidget                 *image;
 	GConfClient               *client;
 
 	if (event->type != GDK_BUTTON_PRESS)
@@ -408,9 +372,6 @@ create_dialog (void)
 	GladeXML     *dialog;
 	GtkSizeGroup *size_group;
 
-	/* register the custom type */
-	(void) mouse_capplet_check_button_get_type ();
-
 	dialog = glade_xml_new (GNOMECC_GLADE_DIR "/gnome-mouse-properties.glade", "mouse_properties_dialog", NULL);
 	if (!dialog)
 		return NULL;
@@ -458,7 +419,6 @@ dialog_response_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changes
 {
 	if (response_id == GTK_RESPONSE_HELP)
 		capplet_help (GTK_WINDOW (dialog),
-			      "user-guide.xml",
 			      "goscustperiph-5");
 	else
 		gtk_main_quit ();
@@ -467,20 +427,26 @@ dialog_response_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changes
 int
 main (int argc, char **argv)
 {
-	GnomeProgram   *program;
 	GConfClient    *client;
 	GladeXML       *dialog;
-	GtkWidget      *dialog_win;
+	GtkWidget      *dialog_win, *w;
+	gchar *start_page = NULL;
 
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
-	program = gnome_program_init ("gnome-mouse-properties", VERSION,
-				      LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
-				      NULL);
+	GOptionContext *context;
+	GOptionEntry cap_options[] = {
+		{"show-page", 'p', G_OPTION_FLAG_IN_MAIN,
+		 G_OPTION_ARG_STRING,
+		 &start_page,
+		 /* TRANSLATORS: don't translate the terms in brackets */
+		 N_("Specify the name of the page to show (general|accessibility)"),
+		 N_("page") },
+		{NULL}
+	};
 
+	context = g_option_context_new (_("- GNOME Mouse Preferences"));
+	g_option_context_add_main_entries (context, cap_options, GETTEXT_PACKAGE);
+	capplet_init (context, &argc, &argv);
+	
 	capplet_init_stock_icons ();
 
 	activate_settings_daemon ();
@@ -498,7 +464,26 @@ main (int argc, char **argv)
 		g_signal_connect (dialog_win, "response",
 				  G_CALLBACK (dialog_response_cb), NULL);
 
-		capplet_set_icon (dialog_win, "gnome-dev-mouse-optical");
+		if (start_page != NULL) {
+			gchar *page_name;
+
+			page_name = g_strconcat (start_page, "_vbox", NULL);
+			g_free (start_page);
+
+			w = WID (page_name);
+			if (w != NULL) {
+				GtkNotebook *nb;
+				gint pindex;
+
+				nb = GTK_NOTEBOOK (WID ("prefs_widget"));
+				pindex = gtk_notebook_page_num (nb, w);
+				if (pindex != -1)
+					gtk_notebook_set_current_page (nb, pindex);
+			}
+			g_free (page_name);
+		}
+
+		capplet_set_icon (dialog_win, "input-mouse");
 		gtk_widget_show (dialog_win);
 
 		gtk_main ();
@@ -507,7 +492,6 @@ main (int argc, char **argv)
 	}
 
 	g_object_unref (client);
-	g_object_unref (program);
 
 	return 0;
 }
